@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: format.c,v 1.33 1999/11/16 21:15:44 nisse Exp $ */
+ * $Id: format.c,v 1.34 1999/11/21 22:36:04 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -645,6 +645,13 @@ int lsh_string_eq_l(const struct lsh_string *a, UINT32 length, const UINT8 *b)
 	   && !memcmp(a->data, b, length));
 }
 
+int lsh_string_prefixp(const struct lsh_string *prefix,
+		       const struct lsh_string *s)
+{
+  return ( (prefix->length <= s->length)
+	   && !memcmp(prefix->data, s->data, prefix->length));
+}
+
 #if 0
 /* FIXME: These functions don't really belong here */
 int lsh_string_cmp(const struct lsh_string *a, const struct lsh_string *b)
