@@ -1,6 +1,6 @@
 /* io.c
  *
- * $Id: io.c,v 1.81 1999/11/16 21:20:45 nisse Exp $ */
+ * $Id: io.c,v 1.82 1999/12/07 00:37:24 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -1252,3 +1252,15 @@ io_write_file(struct io_backend *backend,
 
   return io_write(make_io_fd(backend, fd, e), block_size, c);
 }
+
+struct io_fd *
+io_read_file(struct io_backend *backend,
+	     const char *fname, 
+	     struct exception_handler *e)
+{
+  int fd = open(fname, O_RDONLY);
+  if (fd < 0)
+    return NULL;
+
+  return make_io_fd(backend, fd, e);
+}
