@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: publickey_crypto.h,v 1.27 1999/11/16 21:28:41 nisse Exp $ */
+ * $Id: publickey_crypto.h,v 1.28 1999/11/28 17:30:45 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -27,10 +27,7 @@
 #define LSH_PUBLICKEY_CRYPTO_H_INCLUDED
 
 #include "abstract_crypto.h"
-#include "bignum.h"
-#include "connection.h"
 #include "parse.h"
-#include "sexp.h"
 
 #define GABA_DECLARE
 #include "publickey_crypto.h.x"
@@ -90,12 +87,23 @@ void init_dsa_public(struct dsa_public *public);
 /* parse an ssh keyblob */
 int parse_dsa_public(struct simple_buffer *buffer,
 		     struct dsa_public *public);
+#if 0
 int
 spki_init_dsa_public(struct dsa_public *key,
 		     struct sexp_iterator *i);
+#endif
 
 struct signature_algorithm *make_dsa_algorithm(struct randomness *random);
 
+/* Non spki keys */
+struct dsa_verifier *
+make_ssh_dss_verifier(UINT32 public_length,
+		      UINT8 *public);
+
+struct lsh_string *
+ssh_dss_public_key(struct signer *s);
+
+#if 0
 /* Some support for spki style keys */
 struct dsa_verifier *
 make_dsa_spki_verifier(struct sexp_iterator *i);
@@ -103,6 +111,7 @@ make_dsa_spki_verifier(struct sexp_iterator *i);
 struct dsa_signer *
 make_dsa_spki_signer(struct sexp_iterator *i,
 		     struct randomness *random);
+#endif
 
 
 #if DATAFELLOWS_WORKAROUNDS
