@@ -1,6 +1,6 @@
 /* server_session.c
  *
- * $Id: server_session.c,v 1.19 1999/10/11 21:02:37 nisse Exp $
+ * $Id: server_session.c,v 1.20 1999/11/29 21:32:49 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -328,10 +328,8 @@ struct channel_open *make_open_session(struct unix_user *user,
        ;; (global_requests object alist)
 
        ; Requests specific to session channels 
-       (session_requests object alist)
-
-       ; io_backend, needed for direct_tcpip
-       (backend object io_backend) )) */
+       (session_requests object alist)))
+*/
 
 /* Start an authenticated ssh-connection service */
 static void
@@ -355,12 +353,6 @@ do_login(struct command *s,
      (ATOM_SESSION, make_open_session(user,
 				      closure->session_requests)));
   
-#if 0
-  /* FIXME: Move enabling of direct tcp somewhere else */
-  (ATOM_DIRECT_TCPIP,
-    make_channel_open_direct_tcpip(closure->backend), 
-    -1);
-#endif
 }
 
 /* FIXME: To make this more flexible, we need to have some argument
@@ -368,14 +360,12 @@ do_login(struct command *s,
  * CHANNEL_OPEN, and (ii) for each channel type, the types of
  * channel_requests we want to support. */
 struct command *
-make_server_connection_service(struct alist *session_requests,
-			       struct io_backend *backend)
+make_server_connection_service(struct alist *session_requests)
 {
   NEW(server_connection_service, closure);
 
   closure->super.call = do_login;
   closure->session_requests = session_requests;
-  closure->backend = backend;
 
   return &closure->super;
 }
